/*
 *      The XMMS plugin portion of IMMS
 *      Based heavily on XMMS's song_change plugin
 */


#include <string>

#include <xmms/plugin.h>
#include <xmms/xmmsctrl.h> 
#include <xmms/configfile.h> 

#include <gtk/gtk.h>

#include "imms.h"
#include "version.h"

#define DEFAULT_EMAIL "default@imms.org"
#define DEFAULT_LOGFILE ""

using namespace std;

// Local vars

static Imms *imms = NULL;
static volatile int time_left = -1;
static volatile bool bkl = false;
static int poll_tag = 0, last_small = 0;

static string cur_path = "", last_path = "", next_path = "";

// GUI junk

static GtkWidget *configure_win = NULL, *configure_vbox;
static GtkWidget *email_entry, *logfile_entry;

// Conf vars

gchar *ch_email = NULL, *ch_logfile = NULL;

// C interface

extern "C"
{
    static void init(void);
    static void cleanup(void);
    static void configure(void);
    static gint poll_func(gpointer);
    GeneralPlugin *get_gplugin_info(void);
}

string my_xmms_get_path(int xmms_session, int plpos)
{
    gchar *tmp = xmms_remote_get_playlist_file(xmms_session, plpos);
    string result = tmp;
    g_free(tmp);
    return result;
}

GeneralPlugin imms_gp =
{
    NULL,                   /* handle */
    NULL,                   /* filename */
    -1,                     /* xmms_session */
    NULL,                   /* Description */
    init,
    NULL,
    configure,
    cleanup,
};

GeneralPlugin *get_gplugin_info(void)
{
    imms_gp.description = VERSION;
    return &imms_gp;
}

static void read_config(void)
{
    ConfigFile *cfgfile;

    g_free(ch_email);
    g_free(ch_logfile);
    ch_email = NULL;
    ch_logfile = NULL;

    if ((cfgfile = xmms_cfg_open_default_file()) != NULL)
    {
        xmms_cfg_read_string(cfgfile, "imms", "email", &ch_email);
        xmms_cfg_read_string(cfgfile, "imms", "logfile", &ch_logfile);
        xmms_cfg_free(cfgfile);
    }
}

static void init(void)
{
    read_config();

    if (!imms)
    {
        imms = new Imms();
        imms->setup(ch_email ? ch_email : DEFAULT_EMAIL,
                ch_logfile ? ch_logfile : DEFAULT_LOGFILE);
    }

    poll_tag = gtk_timeout_add(20, poll_func, NULL);
}

static void cleanup(void)
{
    delete imms;
    imms = NULL;

    if (poll_tag)
        gtk_timeout_remove(poll_tag);

    poll_tag = 0;
}

static gint poll_func(gpointer)
{
    if (!xmms_remote_is_playing(xmms_session) &&
            !xmms_remote_is_paused(xmms_session))
        return TRUE;

    if (bkl)
        return TRUE;

    bkl = true;

    if (cur_path != "" && next_path == "")
    {
        int next_plpos = xmms_remote_get_playlist_pos(xmms_session);
        next_path = my_xmms_get_path(xmms_session, next_plpos);
        xmms_remote_playlist_prev(xmms_session);   
        xmms_remote_play(xmms_session);
        bkl = false;
        return TRUE;
    }

    int cur_plpos = xmms_remote_get_playlist_pos(xmms_session);
    cur_path = my_xmms_get_path(xmms_session, cur_plpos);

    if (cur_plpos)
    {
        if (last_path != cur_path)
        {
            last_small = 0;

            last_path = cur_path;
            cur_path = my_xmms_get_path(xmms_session, cur_plpos);
            bool skip = imms->song_changed(cur_path, time_left,
                    next_path != "" && (next_path != cur_path));

            if (skip)
            {
                xmms_remote_playlist_next(xmms_session);
            }
            else if (last_path != "")
            {
                // find out what the next song is going to be
                xmms_remote_play_pause(xmms_session);
                xmms_remote_playlist_next(xmms_session);
                next_path = "";
            }
        }
        else
        {
            int tmp_time = (xmms_remote_get_playlist_time(xmms_session,
                cur_plpos) - xmms_remote_get_output_time(xmms_session)) / 1000;

            // A hack to guard against XMMS reporting random length
            // at the end of a song
            if (tmp_time < 2)
            {
                if (last_small < 5)
                    last_small++;
                time_left = tmp_time < 0 ? time_left : tmp_time;
            }
            else
            {
                if (last_small)
                    last_small--;
                else
                    time_left = tmp_time < 0 ? time_left : tmp_time;
            }
        }
    }

    bkl = false;
    return TRUE;
}

static void configure_ok_cb(gpointer data)
{
        ConfigFile *cfgfile;

        gchar *email = gtk_entry_get_text(GTK_ENTRY(email_entry));
        gchar *logfile = gtk_entry_get_text(GTK_ENTRY(logfile_entry));

        cfgfile = xmms_cfg_open_default_file();
        xmms_cfg_write_string(cfgfile, "imms", "email", email);
        xmms_cfg_write_string(cfgfile, "imms", "logfile", logfile);
        xmms_cfg_write_default_file(cfgfile);
        xmms_cfg_free(cfgfile);

        g_free(ch_email);
        g_free(ch_logfile);
        ch_email = g_strdup(email);
        ch_logfile = g_strdup(logfile);

        if (imms)
        {
            imms->setup(ch_email, ch_logfile);   
        }
        gtk_widget_destroy(configure_win);
}  

static void configure(void)
{
    GtkWidget *email_hbox, *email_label, *logfile_hbox, *logfile_label;
    GtkWidget *song_desc, *end_desc;
    GtkWidget *configure_bbox, *configure_ok, *configure_cancel;
    GtkWidget *song_frame, *song_vbox, *end_frame, *end_vbox;
    gchar *temp;

    if (configure_win)
        return;

    read_config();

    configure_win = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_signal_connect(GTK_OBJECT(configure_win), "destroy",
            GTK_SIGNAL_FUNC(gtk_widget_destroyed), &configure_win);
    gtk_window_set_title(GTK_WINDOW(configure_win), "IMMS Configuration");

    gtk_container_set_border_width(GTK_CONTAINER(configure_win), 10);

    configure_vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_add(GTK_CONTAINER(configure_win), configure_vbox);

    song_frame = gtk_frame_new("IMMS");
    gtk_box_pack_start(GTK_BOX(configure_vbox), song_frame, FALSE, FALSE, 0);
    song_vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_set_border_width(GTK_CONTAINER(song_vbox), 5);
    gtk_container_add(GTK_CONTAINER(song_frame), song_vbox);
    gtk_widget_show(song_frame);
    gtk_widget_show(song_vbox);

    end_frame = gtk_frame_new("Log File");
    gtk_box_pack_start(GTK_BOX(configure_vbox), end_frame, FALSE, FALSE, 0);
    end_vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_set_border_width(GTK_CONTAINER(end_vbox), 5);
    gtk_container_add(GTK_CONTAINER(end_frame), end_vbox);
    gtk_widget_show(end_frame);
    gtk_widget_show(end_vbox);

    temp = g_strdup_printf("The email to identify the user by");
    song_desc = gtk_label_new(temp);
    g_free(temp);
    gtk_label_set_justify(GTK_LABEL(song_desc), GTK_JUSTIFY_LEFT);
    gtk_misc_set_alignment(GTK_MISC(song_desc), 0, 0.5);
    gtk_box_pack_start(GTK_BOX(song_vbox), song_desc, FALSE, FALSE, 0);
    gtk_label_set_line_wrap(GTK_LABEL(song_desc), TRUE);
    gtk_widget_show(song_desc);

    email_hbox = gtk_hbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(song_vbox), email_hbox, FALSE, FALSE, 0);

    email_label = gtk_label_new("Email:");
    gtk_box_pack_start(GTK_BOX(email_hbox), email_label, FALSE, FALSE, 0);
    gtk_widget_show(email_label);

    email_entry = gtk_entry_new();
    if (ch_email)
        gtk_entry_set_text(GTK_ENTRY(email_entry), ch_email);
    else
        gtk_entry_set_text(GTK_ENTRY(email_entry), DEFAULT_EMAIL);
    gtk_widget_set_usize(email_entry, 200, -1);
    gtk_box_pack_start(GTK_BOX(email_hbox), email_entry, TRUE, TRUE, 0);
    gtk_widget_show(email_entry);
    gtk_widget_show(email_hbox);

    end_desc = gtk_label_new("Path to the log file");
    gtk_label_set_justify(GTK_LABEL(end_desc), GTK_JUSTIFY_LEFT);
    gtk_misc_set_alignment(GTK_MISC(end_desc), 0, 0.5);
    gtk_box_pack_start(GTK_BOX(end_vbox), end_desc, FALSE, FALSE, 0);
    gtk_label_set_line_wrap(GTK_LABEL(end_desc), TRUE);
    gtk_widget_show(end_desc);

    logfile_hbox = gtk_hbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(end_vbox), logfile_hbox, FALSE, FALSE, 0);

    logfile_label = gtk_label_new("Path:");
    gtk_box_pack_start(GTK_BOX(logfile_hbox), logfile_label, FALSE, FALSE, 0);
    gtk_widget_show(logfile_label);

    logfile_entry = gtk_entry_new();
    if (ch_logfile)
        gtk_entry_set_text(GTK_ENTRY(logfile_entry), ch_logfile);
    gtk_widget_set_usize(logfile_entry, 200, -1);
    gtk_box_pack_start(GTK_BOX(logfile_hbox), logfile_entry, TRUE, TRUE, 0);
    gtk_widget_show(logfile_entry);
    gtk_widget_show(logfile_hbox);

    configure_bbox = gtk_hbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(configure_bbox), GTK_BUTTONBOX_END);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(configure_bbox), 5);
    gtk_box_pack_start(GTK_BOX(configure_vbox), configure_bbox, FALSE, FALSE, 0);

    configure_ok = gtk_button_new_with_label("Ok");
    gtk_signal_connect(GTK_OBJECT(configure_ok), "clicked",
            GTK_SIGNAL_FUNC(configure_ok_cb), NULL);
    GTK_WIDGET_SET_FLAGS(configure_ok, GTK_CAN_DEFAULT);
    gtk_box_pack_start(GTK_BOX(configure_bbox), configure_ok, TRUE, TRUE, 0);
    gtk_widget_show(configure_ok);
    gtk_widget_grab_default(configure_ok);

    configure_cancel = gtk_button_new_with_label("Cancel");
    gtk_signal_connect_object(GTK_OBJECT(configure_cancel), "clicked",
            GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(configure_win));
    GTK_WIDGET_SET_FLAGS(configure_cancel, GTK_CAN_DEFAULT);
    gtk_box_pack_start(GTK_BOX(configure_bbox), configure_cancel, TRUE, TRUE, 0);
    gtk_widget_show(configure_cancel);
    gtk_widget_show(configure_bbox);
    gtk_widget_show(configure_vbox);
    gtk_widget_show(configure_win);
}
